#ifndef C_GAME_H
#define C_GAME_H

#include <allegro.h>
#include "c_bitmap_factory.h"
#include "c_player.h"
#include "c_enemy.h"
#include "c_projectile.h"
#include "c_particle_system.h"
#include "c_powerup.h"
#include "c_jukebox.h"
#include "c_sfx.h"
#include "c_menu.h"

class c_menu;

/* The score increment per player shot that hits an enemy */
#define SCORE_PER_SHOT_HIT 1

/* The main game state: -
 *  - GS_NONE:     Initial undefined state (game is uninitialised).
 *  - GS_INIT:     Game has been initialised.
 *  - GS_RUN:      Game is running.
 *  - GS_QUIT:     Game is quitting (either from player choice or error).
 *  - GS_SHUTDOWN: Game is being shut down.
 */
enum GAME_STATE { GS_NONE, GS_INIT, GS_RUN, GS_QUIT, GS_SHUTDOWN };

/* The running state of the game (only valid when main state is GS_RUN): -
 *  - GRS_MAINMENU: The main menu is being displayed.
 *  - GRS_PLAYING:  The game is being played.
 *  - GRS_PAUSE:    The game is being played, but paused.
 *  - GRS_GAMEOVER: The game is over, game over screen is being displayed.
 */
enum GAME_RUN_STATE { GRS_MAINMENU, GRS_PLAYING, GRS_PAUSE, GRS_GAMEOVER };

/* struct to store enemies in a linked list */
typedef struct _enemy_list
{
	c_enemy *enemy;
	struct _enemy_list *p_next;
} enemy_list;

/* struct to store projectiles in a linked list */
typedef struct _projectile_list
{
	c_projectile *projectile;
	struct _projectile_list *p_next;
} projectile_list;

/* struct to store particle systems in a linked list */
typedef struct _particle_system_list
{
	c_particle_system *ps;
	struct _particle_system_list *p_next;
} particle_system_list;

/* Class: c_game
 * 
 * Main class used to encapsulate the entire game. Contains the main game loop
 * in c_game::run(). Handles all other objects and is passed to other objects
 * to provide callback functionality (menu events etc).
 */
class c_game
{
private:
	GAME_STATE            m_state;
	GAME_RUN_STATE        m_run_state;
	BITMAP               *m_buf;                // The back buffer (double buffer)
	BITMAP               *m_backdrop;           // The game arena background image
	BITMAP               *m_game_over_backdrop; // The game over screen background image
	BITMAP               *m_mouse_cursor;       // The custom mouse cursor
	c_bitmap_factory     *m_p_bitmap_factory;
	c_player             *m_player;
	enemy_list           *m_p_enemies;
	projectile_list      *m_p_projectiles;
	particle_system_list *m_p_particle_systems;
	c_powerup            *m_p_powerup;
	c_jukebox            *m_p_jukebox;
	c_sfx                *m_p_sfx;
	c_menu               *m_p_menu_main;
	JUKEBOX_TRACK         m_curr_main_track; // The currently chosen gameplay jukebox track
	int                   m_enemy_count;     // Enemy counter (used for spawn probability etc)
	long int              m_frame;           // Frame counter
	int                   m_shots;           // Number of shots fired by the player
	int                   m_shots_hit;       // Number of player shots that hit an enemy
	int                   m_score;           // Player's current score
	int                   m_bound_left;      // The gameplay bounds...
	int                   m_bound_right;     // ...
	int                   m_bound_top;       // ...
	int                   m_bound_bottom;    // ...

	/* Keyboard handler function. Handles keyboard based on game run state. */
	void keyboard_handler(void);
	
	/* Spawns an enemy
	 * Arguments: -
	 *  - x, y:       enemy location (top-left).
	 *  - health:     enemy's starting health.
	 *  - kill_score: the score the player will receive for killing this enemy.
	 *  - dat_file:   file name of the character actor data file.
	 */
	void add_enemy(int x, int y, float health, int kill_score, string dat_file);
	
	/* Updates all enemies in the game (movement, firing etc) */
	void update_enemies(void);
	
	/* Draws all enemies in the game */
	void draw_enemies(void);
	
	/* Updates all projectiles in the game (movement, hit testing etc) */
	void update_projectiles(void);
	
	/* Draws all projectiles in the game */
	void draw_projectiles(void);
	
	/* Checks and handles a single projectile collision
	 * Argument: -
	 *  - projectile: pointer to the c_projectile to check.
	 */
	void projectile_hit_test(c_projectile *projectile);
	
	/* Updates the powerup (decrease lifespan etc) */
	void update_powerup(void);
	
	/* Draws the powerup */
	void draw_powerup(void);
	
	/* Updates the player */
	void update_player(void);
	
	/* Performs a hit test between two rectangles
	 * Arguments: -
	 *  - l1, t1, r1, b1: left, top, right, bottom co-ords for rectangle 1.
	 *  - l2, t2, r2, b2: ...same for rectangle 2.
	 * Return: true if rectangles intersect, false otherwise.
	 */
	static bool rect_hit_test(float l1, float t1, float r1, float b1, float l2, float t2, float r2, float b2);
	
	/* Spawns a particle system
	 * Arguments: -
	 *  - pst:  type of the pasrticle system.
	 *  - x, y: the location of the particle system.
	 *  - vec_dir_x, vec_dir_y: the base velocity vector for the particle system.
	 */
	void add_particle_system(PARTICLE_SYSTEM_TYPE pst, float x, float y, float vec_dir_x, float vec_dir_y);
	
	/* Updates all particle active systems in the game */
	void update_particle_systems(void);
	
	/* Draws all particle systems in the game */
	void draw_particle_systems(void);
public:
	c_game();
	~c_game();

	/* Utility function to cause the thread to sleep for specified milliseconds)
	 * Argument: -
	 *  - ms: the number of milliseconds for the thread to sleep.
	 */
	static void sleep(int ms);
	
	/* Utility function that checks for a key press then blocks until the key is released
	 * Argument: -
	 *  - key_id: the Allegro key ID.
	 * Return: true if key was pressed, false otherwise.
	 */
	static bool key_press(int key_id);
	
	/* Method that will be called from c_menu when the player selects a menu item
	 * Arguments: -
	 *  - menu:    pointer to the calling menu object.
	 *  - item_id: the item in that menu that was selected.
	 */
	void menu_handler(c_menu *menu, int item_id);
	
	/* Spawns a projectile in the game from players and enemies
	 * Arguments: -
	 *  - p_owner:      pointer to the c_character_actor that fired the projectile.
	 *  - damage_type:  the projectile damage type (DT_PLAYER, DT_ENEMY, DT_BOTH).
	 *  - x, y:         the location at which to spawn the projectile.
	 *  - x_vel, y_vel: the projectile's velocity vector.
	 *  - dat_file:     the file name of the actor data file.
	 */
	void fire_projectile(c_character_actor *p_owner, DAMAGE_TYPE damage_type, int x, int y, float x_vel, float y_vel, string dat_file);
	
	/* Return: pointer the the game's bc_bitmap_factory */
	c_bitmap_factory* get_bitmap_factory(void);
	
	/* Return: pointer to the game's c_player */
	c_player* get_player(void);
	
	/* Removes an enemy from the game (i.e. when it becomes dead)
	 * Argument: -
	 *  - p_enemy: pointer to the c_enemy to remove.
	 */
	void remove_enemy(c_enemy *p_enemy);
	
	/* Methods to return the game bounding box co-ordinates */
	int get_bound_top(void);
	int get_bound_left(void);
	int get_bound_bottom(void);
	int get_bound_right(void);
	
	/* Setter fot the game run state */
	void set_run_state(GAME_RUN_STATE state);
	
	/* resets the game (after a game over)
	 * Return: true on success, false on failure
	 */
	bool reset(void);
	
	/* Initialises the game -- must be called before run()
	 * Argument: -
	 *  - f_reset: whether to perform a first-time init (false) or a game reset (true).
	 * Return: true on success, false on failure
	 */
	bool init(bool f_reset);
	
	/* Shuts down the game -- must be called before destroying the game object
	 * Argument: -
	 *  - f_reset: whether to perform a full shutdown (false) or a game reset (true).
	 * Return: true on success, false on failure
	 */
	bool shutdown(bool f_reset);
	
	/* The main game loop -- must be called after init(false) and not after shutdown(false). */
	void run(void);
};

#endif