#include <stdio.h>
#include "c_menu.h"
#include "c_actor.h"
#include "c_sprite.h"

c_menu::c_menu(c_game *game)
{
	_init();
	m_p_game = game;
	
	// Load the cursor actor and set its animation to IDLE
	m_p_actor_cursor = new c_actor(m_p_game, "data/Menu_cursor.dat");
	if (m_p_actor_cursor != NULL)
		m_p_actor_cursor->set_anim("IDLE");
}

c_menu::~c_menu()
{
	if (m_p_actor_heading != NULL)
	{
		delete m_p_actor_heading;
		m_p_actor_heading = NULL;
	}
	if (m_p_actor_cursor != NULL)
	{
		delete m_p_actor_cursor;
		m_p_actor_cursor = NULL;
	}
	if (m_p_backdrop != NULL)
	{
		destroy_bitmap(m_p_backdrop);
		m_p_backdrop = NULL;
	}
}

void c_menu::_init(void)
{
	m_item_count = 0;
	m_curr_item = -1;
	m_p_actor_heading = NULL;
	m_p_actor_cursor  = NULL;
	m_p_backdrop      = NULL;
}

void c_menu::get_item_bounds(int item_id, int *p_x1, int *p_y1, int *p_x2, int *p_y2)
{
	int x_half = m_p_game->get_bound_left() + ((m_p_game->get_bound_right()  - m_p_game->get_bound_left()) / 2);
	
	// If the item ID is invalid, set each co-ordinate to -1
	if (item_id < 0 || item_id >= m_item_count)
	{
		*p_x1 = -1;
		*p_y1 = -1;
		*p_x2 = -1;
		*p_y2 = -1;
		return;
	}
	
	// Calculate left, right, top, bottom values and set int pointers
	*p_x1 = x_half - (text_length(font, m_a_items[item_id].c_str()) / 2);
	*p_x2 = *p_x1 + text_length(font, m_a_items[item_id].c_str());
	*p_y1 = MENU_INITIAL_OFFSET_Y;
	if (m_p_actor_heading != NULL)
		*p_y1 += m_p_actor_heading->get_sprite()->get_tile_height() + MENU_ITEM_PADDING_Y;
	*p_y1 += item_id * MENU_ITEM_PADDING_Y;
	*p_y2 = *p_y1 + text_height(font);
}

void c_menu::load_actor_heading(string filename)
{
	int x_half = m_p_game->get_bound_left() + ((m_p_game->get_bound_right()  - m_p_game->get_bound_left()) / 2);
	
	// Create heading actor, position it and set animation to IDLE
	m_p_actor_heading = new c_actor(m_p_game, filename);
	if (m_p_actor_heading == NULL)
		return;
	m_p_actor_heading->set_x(x_half - (m_p_actor_heading->get_sprite()->get_tile_width() / 2));
	m_p_actor_heading->set_y(MENU_INITIAL_OFFSET_Y);
	m_p_actor_heading->set_anim("IDLE");
}

void c_menu::load_message(string filename)
{
	char buf[1024] = "\0";
	FILE *f = fopen(filename.c_str(), "r");
	
	// Load the message from the text file and assign it to m_message
	if (f)
	{
		fscanf(f, "%1023c", buf);
		m_message.assign(buf);
		fclose(f);
	}
}

c_actor* c_menu::get_actor_heading(void)
{
	return m_p_actor_heading;
}

int c_menu::get_item_count(void)
{
	return m_item_count;
}

int c_menu::add_item(string item_str)
{
	// Stop at MENU_MAX_ITEMS
	if (m_item_count >= MENU_MAX_ITEMS)
		return -1;
	
	m_a_items[m_item_count] = item_str;
	
	m_item_count++;
	
	if (m_curr_item == -1)
		m_curr_item = 0;
	
	return m_item_count;
}

void c_menu::draw(BITMAP *bmp)
{
	int offset_y = MENU_INITIAL_OFFSET_Y;
	int x_half = m_p_game->get_bound_left() + ((m_p_game->get_bound_right()  - m_p_game->get_bound_left()) / 2);
	int y_half = m_p_game->get_bound_top()  + ((m_p_game->get_bound_bottom() - m_p_game->get_bound_top())  / 2);
	string str_temp;
	bool f_lines = false;
	int idx = 0, idx_cr = 0;
	
	// Draw the background
	if (m_p_backdrop != NULL)
		blit(m_p_backdrop, bmp, 0, 0, 0, 0, m_p_backdrop->w, m_p_backdrop->h);
	else
		clear_to_color(bmp, makecol(0, 0, 0));
	
	// Draw the heading if it's loaded
	if (m_p_actor_heading != NULL)
	{
		offset_y += m_p_actor_heading->get_sprite()->get_tile_height() + MENU_ITEM_PADDING_Y;
		m_p_actor_heading->draw(bmp);
	}
	
	// Draw each item
	for (int i = 0; i < m_item_count; i++)
	{
		// If this is the currently selected item, draw the cursor
		if (m_curr_item == i && m_p_actor_cursor != NULL)
		{
			m_p_actor_cursor->set_x(x_half - (text_length(font, m_a_items[i].c_str()) / 2) - 15);
			m_p_actor_cursor->set_y(offset_y);
			m_p_actor_cursor->draw(bmp);
		}
		textprintf_centre_ex(bmp, font, x_half, offset_y, makecol(255, 255, 255), -1, "%s", m_a_items[i].c_str());
		offset_y += MENU_ITEM_PADDING_Y;
	}
	
	// Draw the message if applicable
	if (m_message.length() > 0)
	{
		f_lines = true;
		while (f_lines)
		{
			// Break line at each CR (\n)
			idx_cr = m_message.find("\n", idx);
			if (idx_cr != string::npos)
			{
				str_temp = m_message.substr(idx, idx_cr - idx);
				idx = idx_cr + 1;
			}
			else
			{
				str_temp = m_message.substr(idx, m_message.length() - idx);
				f_lines = false;
			}
			
			textprintf_ex(bmp, font, m_p_game->get_bound_left() + 20, offset_y, makecol(255, 255, 255), -1, "%s", str_temp.c_str());
			offset_y += MENU_ITEM_PADDING_Y;
		}
	}
}

void c_menu::update(void)
{
	int x1 = -1, y1 = -1, x2 = -1, y2 = -1;
	
	if (mouse_needs_poll())
		poll_mouse();
	
	// Select the current menu item based on the current mouse position
	for (int i = 0; i < m_item_count; i++)
	{
		get_item_bounds(i, &x1, &y1, &x2, &y2);
		if (x1 > -1 && y1 > -1 && x2 > -1 && y2 > -1)
		{
			if (mouse_x >= x1 && mouse_x <= x2 && mouse_y >= y1 && mouse_y <= y2)
			{
				m_curr_item = i;
				break;
			}
		}
	}
		
	// If the left mouse button is pressed, check if the cursor is currently over the
	// current item, then if so register a menu select event.
	if (mouse_b & 0x01) // LMB pressed
	{
		get_item_bounds(m_curr_item, &x1, &y1, &x2, &y2);
		if (x1 > -1 && y1 > -1 && x2 > -1 && y2 > -1)
			if (mouse_x >= x1 && mouse_x <= x2 && mouse_y >= y1 && mouse_y <= y2)
				m_p_game->menu_handler(this, m_curr_item);
	}
	
	//Handle key presses
	if (c_game::key_press(KEY_DOWN))
		m_curr_item++;

	if (c_game::key_press(KEY_UP))
		m_curr_item--;
	
	if (c_game::key_press(KEY_ENTER))
		m_p_game->menu_handler(this, m_curr_item);
	
	// Wrap current menu item (previous to first is last and vice versa)
	if (m_curr_item < 0)
		m_curr_item = m_item_count - 1;
	if (m_curr_item >= m_item_count)
		m_curr_item = 0;	
}

void c_menu::load_backdrop(string filename)
{
	m_p_backdrop = load_bitmap(filename.c_str(), NULL);
}
