#include "c_particle_system.h"
#include <math.h>

c_particle_system::c_particle_system(PARTICLE_SYSTEM_TYPE pst, float x, float y, float vec_dir_x, float vec_dir_y)
{
	m_pst = pst;
	m_f_active = false;
	m_x = x;
	m_y = y;
	m_vec_dir_x = vec_dir_x;
	m_vec_dir_y = vec_dir_y;
	m_p_head_particle = NULL;
	
	spawn_particles();
}

c_particle_system::~c_particle_system()
{
	c_particle *p = m_p_head_particle, *p_temp = NULL;
	
	// Delete all particles in the linked list
	while (p != NULL)
	{
		p_temp = p;
		p = p->get_next();
		delete p_temp;
	}
	m_p_head_particle = NULL;
}

void c_particle_system::spawn_particles(void)
{
	c_particle *p_new = NULL;
	float x_vel = 0.0, y_vel = 0.0, mag = 0.0, angle = 0.0, angle_new = 0.0, damping = 0.0;
	unsigned char brightness = 0, r = 0, g = 0, b = 0;
	
	// Convert the particle system's base velocity vector to polar form
	mag = sqrt(m_vec_dir_x * m_vec_dir_x + m_vec_dir_y * m_vec_dir_y);
	
	if (mag != 0)
	{
		// Calculate the angle of the vector (in radians)
		if (m_vec_dir_x > 0 && m_vec_dir_y < 0)
			angle = asin(abs(m_vec_dir_y) / mag);
		else if (m_vec_dir_x < 0 && m_vec_dir_y < 0)
			angle = 0.5 * PI + (asin(abs(m_vec_dir_x) / mag));
		else if (m_vec_dir_x < 0 && m_vec_dir_y > 0)
			angle = PI + (asin(abs(m_vec_dir_y) / mag));
		else if (m_vec_dir_x > 0 && m_vec_dir_y > 0)
			angle = 1.5 * PI + (asin(abs(m_vec_dir_x) / mag));
		else if (m_vec_dir_x == 0 && m_vec_dir_y < 0)
			angle = 0.5 * PI;
		else if (m_vec_dir_x == 0 && m_vec_dir_y > 0)
			angle = 1.5 * PI;
		else if (m_vec_dir_x < 0 && m_vec_dir_y == 0)
			angle = PI;
		else
			angle = 0.0;
	}
	else
	{
		m_vec_dir_x = 0.0;
		m_vec_dir_y = 0.0;
		angle = 0.0;
	}
	
	switch (m_pst)
	{
		case PST_WALL_DEBRIS:			
			for (int i = 0; i < 50; i++)
			{
				// Calculate a random brightness for this particle
				brightness = 64 + (rand() % 192);
				
				// Calculate new angle and particle velocity vector based on random direction variation
				angle_new = angle + (((rand() % (PST_WALL_DEBRIS_ANGLE_DEGREES + 1)) - (PST_WALL_DEBRIS_ANGLE_DEGREES / 2)) * (PI / 180.0));
				x_vel =  mag * cos(angle_new);
				y_vel = -mag * sin(angle_new);
				
				// Scale the velocity vector
				x_vel *= PST_WALL_DEBRIS_VELOCITY;
				y_vel *= PST_WALL_DEBRIS_VELOCITY;
				
				damping = (float)(rand() % 60) / 100.0;
				
				// Create the particle and add it to the head of the list
				p_new = new c_particle(brightness, brightness, brightness, 1, m_x, m_y, x_vel, y_vel, damping, damping, (rand() % 10) + 20);
				if (p_new != NULL)
				{
					p_new->set_next(m_p_head_particle);
					m_p_head_particle = p_new;
				}
			}
			break;
		case PST_BLOOD:
			for (int i = 0; i < 50; i++)
			{
				// Calculate a random brightness for this blood particle
				brightness = 32 + (rand() % 224);
				
				// Calculate a new angle and particle velocity vector based on random direction variation
				angle_new = angle + (((rand() % (PST_BLOOD_ANGLE_DEGREES + 1)) - (PST_BLOOD_ANGLE_DEGREES / 2)) * (PI / 180.0));
				x_vel =  mag * cos(angle_new);
				y_vel = -mag * sin(angle_new);
				
				// Scale the velocity vector
				x_vel *= PST_BLOOD_VELOCITY;
				y_vel *= PST_BLOOD_VELOCITY;
				
				damping = (float)((rand() % 70) + 1) / 100.0;
				
				// Randomly vary the colour of the particle (red ones and some yellowinh ones)
				if (rand() % 5 == 0)
				{
					r = brightness / 2;
					g = brightness / 3;
					b = 0;
				}
				else
				{
					r = brightness;
					g = 0;
					b = 0;
				}
				
				// Create the particle and add it to the head of the list
				p_new = new c_particle(r, g, b, 1, m_x + x_vel, m_y + y_vel, x_vel, y_vel, damping, damping, (rand() % 5) + 10);
				if (p_new != NULL)
				{
					p_new->set_next(m_p_head_particle);
					m_p_head_particle = p_new;
				}
			}
			break;
		case PST_PROJECTILE_TRAIL:
			// Not implemented yet
			break;
	}
}

void c_particle_system::update(void)
{
	c_particle *p = NULL;
	
	m_f_active = false;
	
	if (m_p_head_particle == NULL)
		return;
		
	// Update each particle. If any are still alive, set m_f_active to true
	p = m_p_head_particle;
	while (p != NULL)
	{
		p->update();
		if (!p->is_dead())
			m_f_active = true;
		p = p->get_next();
	}
}

void c_particle_system::draw(BITMAP *bmp_dest)
{
	c_particle *p = NULL;
	
	if (m_p_head_particle == NULL)
		return;
	
	// Draw each particle in the list
	p = m_p_head_particle;
	while (p != NULL)
	{
		p->draw(bmp_dest);
		p = p->get_next();
	}
}

bool c_particle_system::is_active(void)
{
	return m_f_active;
}

PARTICLE_SYSTEM_TYPE c_particle_system::get_type(void)
{
	return m_pst;
}
