#ifndef C_SPRITE_H
#define C_SPRITE_H

#include <allegro.h>
#include <string>
#include "c_game.h"

using namespace std;

/* Three animation types: -
 *  - AT_LOOP: at last frame, animation will return to first frame.
 *  - AT_PINGPONG: at last frame, animation will be reversed, then again at
 *    first frame, etc.
 *  - AT_ONESHOT: at last frame, animation will stop.
 */
enum ANIM_TYPE { AT_LOOP, AT_PINGPONG, AT_ONESHOT };

/* This struct is used to store a linked list of frames */
typedef struct _anim_frame
{
	int x; // Tile x position
	int y; // Tile y position
	_anim_frame *p_next;
	_anim_frame *p_prev;
} anim_frame;

/* This struct stores data for a single animation */
typedef struct _animation
{
	int frame_count;
	ANIM_TYPE anim_type;
	int speed;
	int speed_counter;
	bool f_forward;
	bool f_finished;
	anim_frame *p_head;
	anim_frame *p_tail;
} animation;

/* Class: c_sprite
 * 
 * Encapsulates a sprite together with its animations.
 */
class c_sprite
{
private:
	c_game     *m_p_game;
	BITMAP     *m_bmp;           // Main sprite sheet BITMAP*
	int         m_tile_w;        // The tile width in the bitmap...
	int         m_tile_h;        // ...and height
	anim_frame *m_p_curr_frame;
	int         m_curr_frame_no;
	int         m_curr_anim;
	int         m_anim_count;
	animation   m_anims[16];     // Sprites can have a maximum of 16 animations
	
	/* Performs default initialisations */
	void _init(void);
	
	/* Internal method to step the animation */
	void step_anim(void);
public:
	c_sprite();

	/* Overloaded constructor
	 * Arguments: -
	 *  - game:          pointer to the parent c_game object.
	 *  - filename:      string containing the file name of the sprite sheet bitmap.
	 *  - tile_w/tile_h: the width and height of each tile in the  sprite sheet bitmap.
	 */
	c_sprite(c_game *game, string filename, int tile_w, int tile_h);
	
	~c_sprite();
	
	/* Getters and setters for tile width and height */
	int get_tile_width(void);
	int get_tile_height(void);
	void set_tile_width(int tw);
	void set_tile_height(int th);
	
	/* Loads a sprite sheet bitmap
	 * Arguments: -
	 *  - filename: the file name of the bitmap to load.
	 * Return: true on success, false on failure.
	 */
	bool load_bmp(string filename);
	
	/* Draws the sprite to a BITMAP at a certain location
	 * Arguments: -
	 *  - bmp_dest: the destination BITMAP* to draw to.
	 *  - x, y:     the location (top-left) where the sprite will be drawn).
	 */
	void draw(BITMAP *bmp_dest, int x, int y);
	
	/* Adds an animation to the sprite
	 * Arguments: -
	 *  - anim_type:    the type of animation (see enum above).
	 *  - total_frames: the total number of frames in the animation.
	 *  - speed:        the animation speed. 1 will change frame on every call
	 *                  to draw(), 2 every 2nd call etc.
	 * Return: the new animation ID or -1 on error.
	 */
	int add_anim(ANIM_TYPE anim_type, int total_frames, int speed);
	
	/* Adds a frame to an existing animation
	 * Arguments: -
	 *  - anim_id: the animation ID of the animation to which to add the frame
	 *             (returned from add_anim() above).
	 *  - x, y:    the x and y offset of the tile in the tile sheet (multiplied
	 *             by tile width and height).
	 * Return: true on success, false on failure.
	 */
	bool add_anim_frame(int anim_id, int x, int y);
	
	/* Sets the current animation for the sprite. If the current animation is
	 * already playing, does nothing.
	 * Argument: -
	 *  - anim_id: the ID of the animation to set.
	 * Return: true on success, false on failure.
	 */
	bool set_anim(int anim_id);
	
	/* Gets the ID of the currently playing animation.
	 * Return: the animation ID.
	 */
	int get_anim(void);
	
	/* Indcates whether the current animation has finished playing
	 * Return: true if the animation has finished or has reached the final
	 *         frame at least once (for looped animations), false otherwise.
	 */
	bool is_curr_anim_finished(void);
};

#endif
